@@ -39,17 +39,25 @@ struct Restraints {
       return comp == o.comp ? atom < o.atom : comp < o.comp;
     }
 
-    Atom* get_from(Residue& res1, Residue* res2, char altloc) const {
-      Residue* residue = (comp == 1 || res2 == nullptr ? &res1 : res2);
-      Atom* a = residue->find_atom(atom, altloc);
+    // altloc2 is needed only in rare case when we have a link between
+    // atoms with different altloc (example: 2e7z).
+    Atom* get_from(Residue& res1, Residue* res2, char alt, char altloc2) const {
+      Residue* residue = &res1;
+      if (comp == 2 && res2 != nullptr) {
+        residue = res2;
+        if (altloc2 != '\0')
+          alt = altloc2;
+      }
+      Atom* a = residue->find_atom(atom, alt);
       // Special case: microheterogeneity may have shared atoms only in
       // the first residue. Example: in 1ejg N is shared between PRO and SER.
-      if (a == nullptr && altloc != '\0' && residue->group_idx > 0)
-        a = (residue - residue->group_idx)->find_atom(atom, altloc);
+      if (a == nullptr && alt != '\0' && residue->group_idx > 0)
+        a = (residue - residue->group_idx)->find_atom(atom, alt);
       return a;
     }
-    const Atom* get_from(const Residue& res1, const Residue* res2, char altloc) const {
-      return get_from(const_cast<Residue&>(res1), const_cast<Residue*>(res2), altloc);
+    const Atom* get_from(const Residue& res1, const Residue* res2,
+                         char alt, char alt2) const {
+      return get_from(const_cast<Residue&>(res1), const_cast<Residue*>(res2), alt, alt2);
     }
   };
 
@@ -301,6 +309,25 @@ inline double chiral_abs_volume(double bond1, double bond2, double bond3,
   return mult * std::sqrt(x + y);
 }
 
+inline double chiral_abs_volume_sigma(double bond1, double bond2, double bond3,
+                                      double angle1, double angle2, double angle3,
+                                      double sigb1, double sigb2, double sigb3,
+                                      double siga1, double siga2, double siga3) {
+  double mult = bond1 * bond2 * bond3;
+  auto cosine = [](double a) {return a == 90. ? 0. : std::cos(rad(a));};
+  double cosa1 = cosine(angle1);
+  double cosa2 = cosine(angle2);
+  double cosa3 = cosine(angle3);
+  double x_y = 1 + 2 * cosa1 * cosa2 * cosa3 - sq(cosa1) - sq(cosa2) - sq(cosa3);
+  double varv = x_y * (sq(bond2 * bond3) * sq(sigb1) +
+                       sq(bond1 * bond3) * sq(sigb2) +
+                       sq(bond1 * bond2) * sq(sigb3));
+  varv += sq(mult) / x_y * ((1 - sq(cosa1)) * sq(cosa1 - cosa2 * cosa3) * sq(rad(siga1)) +
+                            (1 - sq(cosa2)) * sq(cosa2 - cosa1 * cosa3) * sq(rad(siga2)) +
+                            (1 - sq(cosa3)) * sq(cosa3 - cosa1 * cosa2) * sq(rad(siga3)));
+  return std::sqrt(varv);
+}
+
 inline double Restraints::chiral_abs_volume(const Restraints::Chirality& ch) const {
   return gemmi::chiral_abs_volume(get_bond(ch.id_ctr, ch.id1).value,
                                   get_bond(ch.id_ctr, ch.id2).value,
@@ -417,7 +444,7 @@ struct ChemComp {
     return const_cast<ChemComp*>(this)->find_atom(atom_id);
   }
   bool has_atom(const std::string& atom_id) const {
-    return find_atom(atom_id) == atoms.end();
+    return find_atom(atom_id) != atoms.end();
   }
 
   int get_atom_index(const std::string& atom_id) const {
@@ -479,23 +506,20 @@ struct ChemComp {
 };
 
 inline BondType bond_type_from_string(const std::string& s) {
-  if (istarts_with(s, "sing"))
-    return BondType::Single;
-  if (istarts_with(s, "doub"))
-    return BondType::Double;
-  if (istarts_with(s, "trip"))
-    return BondType::Triple;
-  if (istarts_with(s, "arom"))
-    return BondType::Aromatic;
-  if (istarts_with(s, "metal"))
-    return BondType::Metal;
-  if (istarts_with(s, "delo") || s == "1.5")
-    return BondType::Deloc;
+  if (s.size() >= 3)
+    switch (ialpha4_id(s.c_str())) {
+      case ialpha4_id("sing"): return BondType::Single;
+      case ialpha4_id("doub"): return BondType::Double;
+      case ialpha4_id("trip"): return BondType::Triple;
+      case ialpha4_id("arom"): return BondType::Aromatic;
+      case ialpha4_id("meta"): return BondType::Metal;
+      case ialpha4_id("delo"): return BondType::Deloc;
+      case ialpha4_id("1.5"):  return BondType::Deloc; // rarely used
+      // program PDB2TNT produces a restraint file with bond type 'coval'
+      case ialpha4_id("cova"):  return BondType::Unspec;
+    }
   if (cif::is_null(s))
     return BondType::Unspec;
-  // program PDB2TNT produces a restraint file with bond type 'coval'
-  if (s == "coval")
-    return BondType::Unspec;
   throw std::out_of_range("Unexpected bond type: " + s);
 }
 
