@@ -154,6 +154,7 @@ struct CifToMtz {
   bool force_unmerged = false;
   std::string title;
   std::vector<std::string> history = { "From gemmi-cif2mtz " GEMMI_VERSION };
+  double wavelength = NAN;
   std::vector<std::string> spec_lines;
 
   Mtz convert_block_to_mtz(const ReflnBlock& rb, std::ostream& out) const {
@@ -166,9 +167,23 @@ struct CifToMtz {
     mtz.cell = rb.cell;
     mtz.spacegroup = rb.spacegroup;
     mtz.add_dataset("HKL_base");
+
     const cif::Loop* loop = rb.refln_loop ? rb.refln_loop : rb.diffrn_refln_loop;
     if (!loop)
       fail("_refln category not found in mmCIF block: " + rb.block.name);
+    bool unmerged = force_unmerged || !rb.refln_loop;
+
+    if (!unmerged) {
+      Mtz::Dataset& ds = mtz.add_dataset("unknown");
+      if (!std::isnan(wavelength))
+        ds.wavelength = wavelength;
+      else if (rb.wavelength_count > 1)
+        out << "Warning: ignoring wavelengths, " << rb.wavelength_count
+            << " are present in block " << rb.block.name << ".\n";
+      else
+        ds.wavelength = rb.wavelength;
+    }
+
     if (verbose)
       out << "Searching tags with known MTZ equivalents ...\n";
     std::vector<int> indices;
@@ -176,8 +191,6 @@ struct CifToMtz {
     std::string tag = loop->tags[0];
     const size_t tag_offset = rb.tag_offset();
 
-    bool unmerged = force_unmerged || !rb.refln_loop;
-
     std::vector<Entry> spec_entries;
     if (!spec_lines.empty()) {
       spec_entries.reserve(spec_lines.size());
@@ -217,9 +230,6 @@ struct CifToMtz {
       col->label = "BATCH";
     }
 
-    if (!unmerged)
-      mtz.add_dataset("unknown").wavelength = rb.wavelength;
-
     // other columns according to the spec
     bool column_added = false;
     for (const Entry& entry : spec_entries) {
