@@ -1,6 +1,6 @@
 // Copyright 2017 Global Phasing Ltd.
 //
-// fail() and unreachable()
+// fail(), unreachable() and __declspec/__attribute__ macros
 
 #ifndef GEMMI_FAIL_HPP_
 #define GEMMI_FAIL_HPP_
@@ -15,11 +15,32 @@
 // warning #2196: routine is both "inline" and "noinline"
 # pragma warning disable 2196
 #endif
+#if defined(__GNUG__) && !defined(__clang__)
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wattributes"
+#endif
 
-#if defined(_MSC_VER)
-# define GEMMI_NOINLINE __declspec(noinline)
+#if defined(__GNUC__) || defined(__clang__)
+# define GEMMI_COLD __attribute__((cold))
+#elif defined(_MSC_VER)
+# define GEMMI_COLD __declspec(noinline)
 #else
-# define GEMMI_NOINLINE __attribute__ ((noinline))
+# define GEMMI_COLD __attribute__((noinline))
+#endif
+
+#if defined(_WIN32)
+# if defined(GEMMI_SHARED)
+#  if defined(GEMMI_BUILD)
+#   define GEMMI_DLL __declspec(dllexport)
+#  else
+#   define GEMMI_DLL __declspec(dllimport)
+#  endif  // GEMMI_BUILD
+# else
+#  define GEMMI_DLL
+# endif  // GEMMI_SHARED
+#else
+# define GEMMI_DLL
+//# define GEMMI_DLL __attribute__((visibility("default")))
 #endif
 
 namespace gemmi {
@@ -34,14 +55,14 @@ void fail(std::string&& str, T&& arg1, Args&&... args) {
 }
 
 [[noreturn]]
-inline GEMMI_NOINLINE void fail(const char* msg) { throw std::runtime_error(msg); }
+inline GEMMI_COLD void fail(const char* msg) { throw std::runtime_error(msg); }
 
 [[noreturn]]
-inline GEMMI_NOINLINE void sys_fail(const std::string& msg) {
+inline GEMMI_COLD void sys_fail(const std::string& msg) {
   throw std::system_error(errno, std::system_category(), msg);
 }
 [[noreturn]]
-inline GEMMI_NOINLINE void sys_fail(const char* msg) {
+inline GEMMI_COLD void sys_fail(const char* msg) {
   throw std::system_error(errno, std::system_category(), msg);
 }
 
@@ -54,5 +75,9 @@ inline GEMMI_NOINLINE void sys_fail(const char* msg) {
 #endif
 }
 
+#if defined(__GNUG__) && !defined(__clang__)
+# pragma GCC diagnostic pop
+#endif
+
 } // namespace gemmi
 #endif
