@@ -23,6 +23,9 @@ constexpr double hc() { return 12398.4197386209; }
 // The Bohr radius (a0) in Angstroms.
 constexpr double bohrradius() { return 0.529177210903; }
 
+// for Mott-Bethe factor
+constexpr double mott_bethe_const() { return 1. / (2 * pi() * pi() * bohrradius()); }
+
 // Used in conversion of ADPs (atomic displacement parameters).
 constexpr double u_to_b() { return 8 * pi() * pi(); }
 
@@ -82,7 +85,9 @@ struct Vec3 {
   double cos_angle(const Vec3& o) const {
     return dot(o) / std::sqrt(length_sq() * o.length_sq());
   }
-  double angle(const Vec3& o) const { return std::acos(cos_angle(o)); }
+  double angle(const Vec3& o) const {
+    return std::acos(std::max(-1., std::min(1., cos_angle(o))));
+  }
   bool approx(const Vec3& o, double epsilon) const {
     return std::fabs(x - o.x) <= epsilon &&
            std::fabs(y - o.y) <= epsilon &&
@@ -95,6 +100,15 @@ struct Vec3 {
 
 inline Vec3 operator*(double d, const Vec3& v) { return v * d; }
 
+/// Rodrigues' rotation formula: rotate vector v about given axis of rotation
+/// (which must be a unit vector) by given angle (in radians).
+inline Vec3 rotate_about_axis(const Vec3& v, const Vec3& axis, double theta) {
+  double sin_theta = std::sin(theta);
+  double cos_theta = std::cos(theta);
+  return v * cos_theta + axis.cross(v) * sin_theta +
+         axis * (axis.dot(v) * (1 - cos_theta));
+}
+
 struct Mat33 {
   double a[3][3] = { {1.,0.,0.}, {0.,1.,0.}, {0.,0.,1.} };
 
@@ -109,6 +123,10 @@ struct Mat33 {
         double c1, double c2, double c3)
   : a{{a1, a2, a3}, {b1, b2, b3}, {c1, c2, c3}} {}
 
+  static Mat33 from_columns(const Vec3& c1, const Vec3& c2, const Vec3& c3) {
+    return Mat33(c1.x, c2.x, c3.x, c1.y, c2.y, c3.y, c1.z, c2.z, c3.z);
+  }
+
   Vec3 row_copy(int i) const {
     if (i < 0 || i > 2)
       throw std::out_of_range("Mat33 row index must be 0, 1 or 2.");
@@ -384,104 +402,6 @@ struct Box {
   void add_margin(double m) { add_margins(Pos(m, m, m)); }
 };
 
-// popular single-pass algorithm for calculating variance and mean
-struct Variance {
-  int n = 0;
-  double sum_sq = 0.;
-  double mean_x = 0.;
-
-  Variance() = default;
-  template <typename T> Variance(T begin, T end) : Variance() {
-    for (auto i = begin; i != end; ++i)
-      add_point(*i);
-  }
-  void add_point(double x) {
-    ++n;
-    double dx = x - mean_x;
-    mean_x += dx / n;
-    sum_sq += dx * (x - mean_x);
-  }
-  double for_sample() const { return sum_sq / (n - 1); }
-  double for_population() const { return sum_sq / n; }
-};
-
-struct Covariance : Variance {
-  double mean_y = 0.;
-  void add_point(double x, double y) {
-    ++n;
-    double dx = x - mean_x;
-    mean_x += dx / n;
-    mean_y += (y - mean_y) / n;
-    sum_sq += dx * (y - mean_y);
-  }
-};
-
-struct Correlation {
-  int n = 0;
-  double sum_xx = 0.;
-  double sum_yy = 0.;
-  double sum_xy = 0.;
-  double mean_x = 0.;
-  double mean_y = 0.;
-  void add_point(double x, double y) {
-    ++n;
-    double weight = (double)(n - 1) / n;
-    double dx = x - mean_x;
-    double dy = y - mean_y;
-    sum_xx += weight * dx * dx;
-    sum_yy += weight * dy * dy;
-    sum_xy += weight * dx * dy;
-    mean_x += dx / n;
-    mean_y += dy / n;
-  }
-  double coefficient() const { return sum_xy / std::sqrt(sum_xx * sum_yy); }
-  double x_variance() const { return sum_xx / n; }
-  double y_variance() const { return sum_yy / n; }
-  double covariance() const { return sum_xy / n; }
-  double mean_ratio() const { return mean_y / mean_x; }
-  // the regression line
-  double slope() const { return sum_xy / sum_xx; }
-  double intercept() const { return mean_y - slope() * mean_x; }
-};
-
-
-struct DataStats {
-  double dmin = NAN;
-  double dmax = NAN;
-  double dmean = NAN;
-  double rms = NAN;
-  size_t nan_count = 0;
-};
-
-template<typename T>
-DataStats calculate_data_statistics(const std::vector<T>& data) {
-  DataStats stats;
-  double sum = 0;
-  double sq_sum = 0;
-  stats.dmin = INFINITY;
-  stats.dmax = -INFINITY;
-  for (double d : data) {
-    if (std::isnan(d)) {
-      stats.nan_count++;
-      continue;
-    }
-    sum += d;
-    sq_sum += d * d;
-    if (d < stats.dmin)
-      stats.dmin = d;
-    if (d > stats.dmax)
-      stats.dmax = d;
-  }
-  if (stats.nan_count != data.size()) {
-    stats.dmean = sum / (data.size() - stats.nan_count);
-    stats.rms = std::sqrt(sq_sum / (data.size() - stats.nan_count) - stats.dmean * stats.dmean);
-  } else {
-    stats.dmin = NAN;
-    stats.dmax = NAN;
-  }
-  return stats;
-}
-
 // internally used functions
 namespace impl {
 // MSVC is missing isnan(IntegralType), so we define is_nan as a replacement
