@@ -8,11 +8,13 @@
 #define GEMMI_MERGE_HPP_
 
 #include <cassert>
+#include "atof.hpp"     // for fast_from_chars
 #include "symmetry.hpp"
 #include "unitcell.hpp"
 #include "util.hpp"     // for vector_remove_if
 #include "mtz.hpp"      // for Mtz
 #include "refln.hpp"    // for ReflnBlock
+#include "stats.hpp"    // for Correlation
 #include "xds_ascii.hpp" // for XdsAscii
 
 namespace gemmi {
@@ -27,9 +29,75 @@ inline std::string miller_str(const Miller& hkl) {
   return s;
 }
 
-struct Intensities {
-  enum class Type { None, Unmerged, Mean, Anomalous };
+inline bool parse_voigt_notation(const char* start, const char* end, SMat33<double>& b) {
+  bool first = true;
+  for (double* u : {&b.u11, &b.u22, &b.u33, &b.u23, &b.u13, &b.u12}) {
+    if (*start != (first ? '(' : ','))
+      return false;
+    first = false;
+    auto result = fast_from_chars(++start, end, *u);
+    if (result.ec != std::errc())
+      return false;
+    start = skip_blank(result.ptr);
+  }
+  return *start == ')';
+}
 
+inline bool read_staraniso_b_from_mmcif(const cif::Block& block, SMat33<double>& output) {
+  // read what is written by MtzToCif::write_staraniso_b()
+  cif::Table table = const_cast<cif::Block*>(&block)->find(
+      "_reflns.pdbx_aniso_B_tensor_eigen",
+      {"value_1", "value_2", "value_3",
+       "vector_1_ortho[1]", "vector_1_ortho[2]", "vector_1_ortho[3]",
+       "vector_2_ortho[1]", "vector_2_ortho[2]", "vector_2_ortho[3]",
+       "vector_3_ortho[1]", "vector_3_ortho[2]", "vector_3_ortho[3]"});
+  if (!table.ok())
+    return false;
+  cif::Table::Row row = table.one();
+  using cif::as_number;
+  double eigval[3] = {as_number(row[0]), as_number(row[1]), as_number(row[2])};
+  double min_val = std::min(std::min(eigval[0], eigval[1]), eigval[2]);
+  Mat33 mat(as_number(row[3]), as_number(row[6]), as_number(row[9]),
+            as_number(row[4]), as_number(row[7]), as_number(row[10]),
+            as_number(row[5]), as_number(row[8]), as_number(row[11]));
+  Vec3 diag(eigval[0] - min_val, eigval[1] - min_val, eigval[2] - min_val);
+  // If the columns of mat are an orthonomal basis, mat^−1==mat^T.
+  // But just in case it isn't, we use mat^-1 here.
+  Mat33 t = mat.multiply_by_diagonal(diag).multiply(mat.inverse());
+  // t is a symmetric tensor, so we return only 6 numbers
+  output = {t[0][0], t[1][1], t[2][2], t[0][1], t[0][2], t[1][2]};
+  return true;
+}
+
+// returns STARANISO version or empty string
+inline std::string read_staraniso_b_from_mtz(const Mtz& mtz, SMat33<double>& output) {
+  std::string version;
+  size_t hlen = mtz.history.size();
+  for (size_t i = 0; i != hlen; ++i)
+    if (mtz.history[i].find("STARANISO") != std::string::npos) {
+      size_t version_pos = mtz.history[i].find("version:");
+      if (version_pos != std::string::npos)
+        version = read_word(mtz.history[i].c_str() + version_pos + 8);
+      else
+        version = "?";
+      // StarAniso 2.3.74 (24-Apr-2021) and later write B tensor in history
+      for (size_t j = i+1; j < std::min(i+4, hlen); ++j) {
+        const std::string& line = mtz.history[j];
+        if (starts_with(line, "B=(")) {
+          if (!parse_voigt_notation(line.c_str() + 2,
+                                    line.c_str() + line.size(),
+                                    output))
+            fail("failed to parse tensor Voigt notation: " + line);
+          break;
+        }
+      }
+      break;
+    }
+  return version;
+}
+
+
+struct Intensities {
   struct Refl {
     Miller hkl;
     short isign;  // 1 for I(+), -1 for I(-), 0 for mean
@@ -46,6 +114,23 @@ struct Intensities {
       if (isign > 0) return "I(+)";
       return "I(-)";
     }
+
+    std::string hkl_label() const {
+      std::string s = intensity_label();
+      s += ' ';
+      s += miller_str(hkl);
+      return s;
+    };
+  };
+
+  struct AnisoScaling {
+    SMat33<double> b = {0., 0., 0., 0., 0., 0.};
+
+    bool ok() const { return !b.all_zero(); }
+    double scale(const Miller& hkl, const UnitCell& cell) const {
+      Vec3 s = cell.frac.mat.left_multiply(Vec3(hkl[0], hkl[1], hkl[2]));
+      return std::exp(0.5 * b.r_u_r(s));
+    }
   };
 
   std::vector<Refl> data;
@@ -53,14 +138,16 @@ struct Intensities {
   UnitCell unit_cell;
   double unit_cell_rmsd[6] = {0., 0., 0., 0., 0., 0.};
   double wavelength;
-  Type type = Type::None;
+  DataType type = DataType::Unknown;
+  AnisoScaling staraniso_b;
 
-  static const char* type_str(Type itype) {
-    switch (itype) {
-      case Type::None: return "n/a";
-      case Type::Unmerged: return "I";
-      case Type::Mean: return "<I>";
-      case Type::Anomalous: return "I+/I-";
+
+  static const char* type_str(DataType data_type) {
+    switch (data_type) {
+      case DataType::Unknown: return "n/a";
+      case DataType::Unmerged: return "I";
+      case DataType::Mean: return "<I>";
+      case DataType::Anomalous: return "I+/I-";
     }
     unreachable();
   }
@@ -113,11 +200,11 @@ struct Intensities {
 
   void sort() { std::sort(data.begin(), data.end()); }
 
-  void merge_in_place(Type itype) {
-    type = itype;
+  void merge_in_place(DataType data_type) {
+    type = data_type;
     if (data.empty())
       return;
-    if (itype == Type::Mean)
+    if (data_type == DataType::Mean)
       // discard signs so that merging produces Imean
       for (Refl& refl : data)
         refl.isign = 0;
@@ -196,7 +283,7 @@ struct Intensities {
     // Aimless >=0.7.6 (from 2021) has an option to output unmerged file
     // with original indices instead of reduced indices, with all ISYM = 1.
     switch_to_asu_indices();
-    type = Type::Unmerged;
+    type = DataType::Unmerged;
   }
 
   void read_mean_intensities_from_mtz(const Mtz& mtz) {
@@ -209,7 +296,7 @@ struct Intensities {
     copy_metadata(mtz);
     wavelength = mtz.dataset(col->dataset_id).wavelength;
     read_data(MtzDataProxy{mtz}, col->idx, sigma_idx);
-    type = Type::Mean;
+    type = DataType::Mean;
   }
 
   void read_anomalous_intensities_from_mtz(const Mtz& mtz, bool check_complete=false) {
@@ -229,7 +316,7 @@ struct Intensities {
     copy_metadata(mtz);
     wavelength = mtz.dataset(colp->dataset_id).wavelength;
     read_anomalous_data(MtzDataProxy{mtz}, mean_idx, value_idx, sigma_idx);
-    type = Type::Anomalous;
+    type = DataType::Anomalous;
   }
 
   void read_merged_intensities_from_mtz(const Mtz& mtz) {
@@ -239,18 +326,18 @@ struct Intensities {
       read_mean_intensities_from_mtz(mtz);
   }
 
-  void read_mtz(const Mtz& mtz, Type itype) {
-    switch (itype) {
-      case Type::Unmerged:
+  void read_mtz(const Mtz& mtz, DataType data_type) {
+    switch (data_type) {
+      case DataType::Unmerged:
         read_unmerged_intensities_from_mtz(mtz);
         break;
-      case Type::Mean:
+      case DataType::Mean:
         read_mean_intensities_from_mtz(mtz);
         break;
-      case Type::Anomalous:
+      case DataType::Anomalous:
         read_anomalous_intensities_from_mtz(mtz);
         break;
-      case Type::None:
+      case DataType::Unknown:
         assert(0);
         break;
     }
@@ -263,7 +350,7 @@ struct Intensities {
     wavelength = rb.wavelength;
     read_data(ReflnDataProxy(rb), value_idx, sigma_idx);
     switch_to_asu_indices();
-    type = Type::Unmerged;
+    type = DataType::Unmerged;
   }
 
   void read_mean_intensities_from_mmcif(const ReflnBlock& rb) {
@@ -272,7 +359,7 @@ struct Intensities {
     copy_metadata(rb);
     wavelength = rb.wavelength;
     read_data(ReflnDataProxy(rb), value_idx, sigma_idx);
-    type = Type::Mean;
+    type = DataType::Mean;
   }
 
   void read_anomalous_intensities_from_mmcif(const ReflnBlock& rb,
@@ -287,7 +374,7 @@ struct Intensities {
     copy_metadata(rb);
     wavelength = rb.wavelength;
     read_anomalous_data(ReflnDataProxy(rb), mean_idx, value_idx, sigma_idx);
-    type = Type::Anomalous;
+    type = DataType::Anomalous;
   }
 
   void read_merged_intensities_from_mmcif(const ReflnBlock& rb) {
@@ -318,21 +405,21 @@ struct Intensities {
       r.value *= r.value;
       r.sigma *= 2 * r.value;
     }
-    type = Type::Mean;
+    type = DataType::Mean;
   }
 
-  void read_mmcif(const ReflnBlock& rb, Type itype) {
-    switch (itype) {
-      case Type::Unmerged:
+  void read_mmcif(const ReflnBlock& rb, DataType data_type) {
+    switch (data_type) {
+      case DataType::Unmerged:
         read_unmerged_intensities_from_mmcif(rb);
         break;
-      case Type::Mean:
+      case DataType::Mean:
         read_mean_intensities_from_mmcif(rb);
         break;
-      case Type::Anomalous:
+      case DataType::Anomalous:
         read_anomalous_intensities_from_mmcif(rb);
         break;
-      case Type::None:
+      case DataType::Unknown:
         break;
     }
   }
@@ -345,7 +432,16 @@ struct Intensities {
     for (const XdsAscii::Refl& in : xds.data)
       add_if_valid(in.hkl, 0, in.iobs, in.sigma);
     switch_to_asu_indices();
-    type = Type::Unmerged;
+    type = DataType::Unmerged;
+  }
+
+  // returns STARANISO version or empty string
+  std::string take_staraniso_b_from_mtz(const Mtz& mtz) {
+    return read_staraniso_b_from_mtz(mtz, staraniso_b.b);
+  }
+
+  bool take_staraniso_b_from_mmcif(const cif::Block& block) {
+    return read_staraniso_b_from_mmcif(block, staraniso_b.b);
   }
 
 private:
