@@ -13,12 +13,15 @@ void add_meta(pybind11::module& m); // meta.cpp
 void add_mol(pybind11::module& m); // mol.cpp
 void add_mtz(pybind11::module& m); // mtz.cpp
 void add_cif(pybind11::module& cif); // cif.cpp
-void add_read_structure(pybind11::module& m); // read.cpp
 void add_cif_read(pybind11::module& cif); // read.cpp
+void add_read_structure(pybind11::module& m); // read.cpp
+void add_small(pybind11::module& m); // read.cpp
+void add_chemcomp(pybind11::module& m); // chemcomp.cpp
 void add_monlib(pybind11::module& m); // monlib.cpp
+void add_topo(pybind11::module& m); // topo.cpp
+void add_refine(pybind11::module& m); // refine.cpp
 void add_alignment(pybind11::module& m); // align.cpp
 void add_scaling(pybind11::module& m); // scaling.cpp
-void add_select(pybind11::module& m); // align.cpp
 void add_search(pybind11::module& m); // search.cpp
 void add_sf(pybind11::module& m); // sf.cpp
 void add_custom(pybind11::module& m); // custom.cpp
