@@ -35,6 +35,14 @@ void add_monlib(py::module& m) {
                ChemComp::group_str(self.group) + ">";
     });
 
+  py::class_<ChemMod::AtomMod>(chemmod, "AtomMod")
+    .def_readwrite("func", &ChemMod::AtomMod::func) 
+    .def_readwrite("old_id", &ChemMod::AtomMod::old_id) 
+    .def_readwrite("new_id", &ChemMod::AtomMod::new_id) 
+    .def_readwrite("el", &ChemMod::AtomMod::el) 
+    .def_readwrite("charge", &ChemMod::AtomMod::charge) 
+    .def_readwrite("chem_type", &ChemMod::AtomMod::chem_type) 
+  ;
   chemlink
     .def(py::init<>())
     .def_readwrite("id", &ChemLink::id)
@@ -52,14 +60,17 @@ void add_monlib(py::module& m) {
     .def_readwrite("name", &ChemMod::name)
     .def_readwrite("comp_id", &ChemMod::comp_id)
     .def_readwrite("group_id", &ChemMod::group_id)
+    .def_readwrite("atom_mods", &ChemMod::atom_mods)
     .def_readwrite("rt", &ChemMod::rt)
     .def("__repr__", [](const ChemMod& self) {
         return "<gemmi.ChemMod " + self.id + ">";
     });
 
+  py::class_<EnerLib>(m, "EnerLib");
   py::class_<MonLib>(m, "MonLib")
     .def(py::init<>())
     .def_readonly("monomer_dir", &MonLib::monomer_dir)
+    .def_readonly("ener_lib", &MonLib::ener_lib)
     .def_readonly("monomers", &MonLib::monomers)
     .def_readonly("links", &MonLib::links)
     .def_readonly("modifications", &MonLib::modifications)
@@ -67,26 +78,25 @@ void add_monlib(py::module& m) {
          py::return_value_policy::reference_internal)
     .def("get_mod", &MonLib::get_mod, py::arg("name"),
          py::return_value_policy::reference_internal)
-    .def("match_link", [](const MonLib &self,
-                          const Residue& res1, const std::string& atom1,
-                          const Residue& res2, const std::string& atom2,
-                          char alt, double min_bond_sq) {
+    .def("match_link", &MonLib::match_link,
+         py::arg("res1"), py::arg("atom1"), py::arg("alt1"),
+         py::arg("res2"), py::arg("atom2"), py::arg("alt2"),
+         py::arg("min_bond_sq")=0.,
+         py::return_value_policy::reference_internal)
+    .def("test_link", [](const MonLib& self, const ChemLink& link,
+                         const std::string& res1, const std::string& atom1,
+                         const std::string& res2, const std::string& atom2) {
       const ChemComp::Aliasing* aliasing1 = nullptr;
       const ChemComp::Aliasing* aliasing2 = nullptr;
-      return self.match_link(res1, atom1, res2, atom2, alt, min_bond_sq,
-                             &aliasing1, &aliasing2);
-    }, py::arg("res1"), py::arg("atom1"),
-       py::arg("res2"), py::arg("atom2"), py::arg("altloc"),
-       py::arg("min_bond_sq")=0.,
-       py::return_value_policy::reference_internal)
+      bool match = (!link.rt.bonds.empty() &&
+                    self.link_side_matches_residue(link.side1, res1, &aliasing1) &&
+                    self.link_side_matches_residue(link.side2, res2, &aliasing2) &&
+                    atom_match_with_alias(link.rt.bonds[0].id1.atom, atom1, aliasing1) &&
+                    atom_match_with_alias(link.rt.bonds[0].id2.atom, atom2, aliasing2));
+      return py::make_tuple(match, aliasing1, aliasing2);
+    }, py::arg("link"), py::arg("res1"), py::arg("atom1"), py::arg("res2"), py::arg("atom2"))
     .def("add_monomer_if_present", &MonLib::add_monomer_if_present)
-    .def("insert_chemcomps", &MonLib::insert_chemcomps)
-    .def("insert_chemlinks", &MonLib::insert_chemlinks)
-    .def("insert_chemmods", &MonLib::insert_chemmods)
-    // deprecated
-    .def("insert_comp_list", [](MonLib &self, const cif::Document &doc) {
-        insert_comp_list(doc, self.cc_groups);
-    })
+    .def("read_monomer_doc", &MonLib::read_monomer_doc)
     .def("read_monomer_cif", [](MonLib& self, const std::string& path) {
       return self.read_monomer_cif(path, gemmi::read_cif_gz);
     })
@@ -94,6 +104,9 @@ void add_monlib(py::module& m) {
                                 const std::vector<std::string>& resnames) {
       return self.read_monomer_lib(monomer_dir, resnames, gemmi::read_cif_gz);
     })
+    .def("find_ideal_distance", [](const MonLib& self, CRA &cra1, CRA cra2) {
+      return self.find_ideal_distance(cra1, cra2);
+    })
     .def("path", &MonLib::path, py::arg("code")=std::string())
     .def("__repr__", [](const MonLib& self) {
         return "<gemmi.MonLib with " +
