@@ -58,6 +58,7 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
     .def("nonzero", &M::nonzero)
     .def("determinant", &M::determinant)
     .def("inverse", &M::inverse)
+    .def("scaled", &M::template scaled<T>)
     .def("added_kI", &M::added_kI)
     .def("r_u_r", (double (M::*)(const Vec3&) const) &M::r_u_r)
     .def("r_u_r", [](const M& self, py::array_t<int> arr) {
@@ -101,6 +102,7 @@ void add_unitcell(py::module& m) {
     .def_readwrite("x", &Vec3::x)
     .def_readwrite("y", &Vec3::y)
     .def_readwrite("z", &Vec3::z)
+    .def("normalized", &Vec3::normalized)
     .def("dot", &Vec3::dot)
     .def("cross", &Vec3::cross)
     .def("length", &Vec3::length)
@@ -193,12 +195,6 @@ void add_unitcell(py::module& m) {
     .def("approx", &Transform::approx, py::arg("other"), py::arg("epsilon"));
   transform.attr("__matmul__") = transform.attr("combine");
 
-  py::class_<Correlation>(m, "Correlation")
-    .def_readonly("n", &Correlation::n)
-    .def("coefficient", &Correlation::coefficient)
-    .def("mean_ratio", &Correlation::mean_ratio)
-    ;
-
   py::class_<Position, Vec3>(m, "Position")
     .def(py::init<double,double,double>())
     .def(py::init<const Vec3&>())
@@ -220,7 +216,9 @@ void add_unitcell(py::module& m) {
     });
   py::class_<Fractional, Vec3>(m, "Fractional")
     .def(py::init<double,double,double>())
+    .def(py::init<const Vec3&>())
     .def("wrap_to_unit", &Fractional::wrap_to_unit)
+    .def("wrap_to_zero", &Fractional::wrap_to_zero)
     .def("__getitem__", (double (Fractional::*)(int) const) &Fractional::at)
     .def(py::self + py::self)
     .def(py::self - py::self)
